import{j as o}from"./@radix-AvWw-1nd.js";import{S as r}from"./Tabs-AuhCyzle.js";import{R as m}from"./RunsBody-Cj4sIqQB.js";import{R as i}from"./RunSelector-BVKB4Z8F.js";import{S as p}from"./StackList-5UB8LoEq.js";import{b as s}from"./@react-router-BUo5vhN4.js";import"./index-DPjvk73v.js";import"./@tanstack-CcI3lvwB.js";import"./@reactflow-BHoFKFSZ.js";import"./chevron-down-A3PXOshS.js";import"./NestedCollapsible-Da-k0Mff.js";import"./type-guards-CaeD8wHO.js";import"./CodeSnippet-DUkCnBpQ.js";import"./CollapsibleCard-B5-5Plnd.js";import"./DisplayDate-C5Aw-Yca.js";import"./InlineAvatar-Bin9UPKJ.js";import"./refresh-CupyU1Vs.js";import"./SearchField-DjAOZic5.js";import"./chevron-right-double-zKz7rAaU.js";import"./index-BVVKxTWC.js";import"./all-pipeline-runs-query-COvsm3bC.js";import"./ExecutionStatus-CD8Vj7sp.js";import"./check-circle-DyCCYTA0.js";import"./trash-B_JgTgqd.js";import"./DeleteAlertDialog-BgTZbbAt.js";import"./zod-CRNUMWWg.js";import"./index.esm-cf-8NBxV.js";import"./AlertDialogDropdownItem-D7KZcPFw.js";import"./delete-run-Do3XyF4W.js";import"./plus-CoKtHiA9.js";import"./index-D-n6tspq.js";import"./stack-detail-query-omCumL7U.js";import"./components-Br2ezRib.js";import"./NumberBox-BvBJYxCu.js";import"./ComponentBadge-FrujKBC6.js";import"./ComponentIcon-Dx5fBrDX.js";import"./layout-3_rgDUxf.js";import"./rocket-k68ONPDS.js";import"./DialogItem-CNWLiJcc.js";import"./Infobox-BHEdNmME.js";function U(){const{componentId:t}=s();return o.jsx("div",{className:"@container",children:o.jsx(r,{isPanel:!1,stacksTabContent:o.jsx(p,{fixedQueryParams:{component_id:t}}),runsTabContent:o.jsx(i,{children:o.jsx(m,{fixedQueryParams:{stack_component:t}})}),componentId:t})})}export{U as default};
